@@ -13,8 +13,24 @@ pub fn transcoder_decorator(ast: &syn::MacroInput) -> Result<quote::Tokens, &str
     }
     let has_lifetime = num_lifetimes == 1;
 
-    let decoder = gen_decoder(&ast.ident, struct_fields, has_lifetime);
-    let encoder = gen_encoder(&ast.ident, struct_fields, has_lifetime);
+    let field_atoms: Vec<Tokens> = struct_fields.iter().map(|field| {
+        let ident = field.clone().ident.unwrap();
+        let ident_str = ident.to_string();
+
+        let atom_fun = Ident::new(format!("atom_{}", ident_str));
+
+        quote! {
+            atom #atom_fun = #ident_str;
+        }
+    }).collect();
+    let atom_defs = quote! {
+        rustler_atoms! {
+            #(#field_atoms)*
+        }
+    };
+
+    let decoder = gen_decoder(&ast.ident, struct_fields, &atom_defs, has_lifetime);
+    let encoder = gen_encoder(&ast.ident, struct_fields, &atom_defs, has_lifetime);
 
     Ok(quote! {
         #decoder
@@ -22,25 +38,15 @@ pub fn transcoder_decorator(ast: &syn::MacroInput) -> Result<quote::Tokens, &str
     })
 }
 
-pub fn gen_decoder(struct_name: &Ident, fields: &Vec<Field>, has_lifetime: bool) -> Tokens {
+pub fn gen_decoder(struct_name: &Ident, fields: &Vec<Field>, atom_defs: &Tokens, has_lifetime: bool) -> Tokens {
     let field_defs: Vec<Tokens> = fields.iter().map(|field| {
         let ident = field.clone().ident.unwrap();
         let ident_str = ident.to_string();
+
+        let atom_fun = Ident::new(format!("atom_{}", ident_str));
+
         quote! {
-            #ident: {
-                match rustler::NifDecoder::decode(
-                    match rustler::map::get_map_value(
-                        term,
-                        rustler::atom::get_atom_init(#ident_str).to_term(env))
-                    {
-                        Some(term) => term,
-                        None => return Err(rustler::NifError::BadArg),
-                    })
-                {
-                    Ok(res) => res,
-                    Err(err) => return Err(err),
-                }
-            }
+            #ident: rustler::NifDecoder::decode(term.map_get(#atom_fun().encode(env))?)?
         }
     }).collect();
 
@@ -53,6 +59,8 @@ pub fn gen_decoder(struct_name: &Ident, fields: &Vec<Field>, has_lifetime: bool)
     quote! {
         impl<'a> rustler::NifDecoder<'a> for #struct_type {
             fn decode(term: rustler::NifTerm<'a>) -> Result<Self, rustler::NifError> {
+                #atom_defs
+
                 let env = term.get_env();
                 Ok(#struct_name { #(#field_defs),* })
             }
@@ -60,16 +68,15 @@ pub fn gen_decoder(struct_name: &Ident, fields: &Vec<Field>, has_lifetime: bool)
     }
 }
 
-pub fn gen_encoder(struct_name: &Ident, fields: &Vec<Field>, has_lifetime: bool) -> Tokens {
+pub fn gen_encoder(struct_name: &Ident, fields: &Vec<Field>, atom_defs: &Tokens, has_lifetime: bool) -> Tokens {
     let field_defs: Vec<Tokens> = fields.iter().map(|field| {
         let field_ident = field.clone().ident.unwrap();
         let field_ident_str = field_ident.to_string();
+
+        let atom_fun = Ident::new(format!("atom_{}", field_ident_str));
+
         quote! {
-            map = rustler::map::map_put(
-                map,
-                rustler::atom::get_atom_init(#field_ident_str).to_term(env),
-                self.#field_ident.encode(env)
-                ).unwrap();
+            map = map.map_put(#atom_fun().encode(env), self.#field_ident.encode(env)).ok().unwrap();
         }
     }).collect();
 
@@ -80,13 +87,12 @@ pub fn gen_encoder(struct_name: &Ident, fields: &Vec<Field>, has_lifetime: bool)
     };
 
     quote! {
-        impl<'a> rustler::codegen_runtime::GeneratedNifTranscoder<'a, rustler::codegen_runtime::MapType> {
-
-        }
         impl<'b> rustler::NifEncoder for #struct_type {
             fn encode<'a>(&self, env: rustler::NifEnv<'a>) -> rustler::NifTerm<'a> {
-                let mut map = rustler::map::map_new(env);
-                #(field_defs)*
+                #atom_defs
+
+                let mut map = rustler::types::map::map_new(env);
+                #(#field_defs)*
                 map
             }
         }
